#include <string.h>
#include <misc.h>
#include <dialog.h>
#include <translat.h>

// #Specbeg: dialog / sample code / records
/*
	This shows how to handle table of records. This is pretty much
	what linuxconf is doing all around.
*/

class PERSON: public ARRAY_OBJ{
public:
	SSTRING name;
	SSTRING surname;
	SSTRING tel;
	PERSON(const char *_name, const char *_surname, const char *_tel){
		name.setfrom (_name);
		surname.setfrom (_surname);
		tel.setfrom (_tel);
	}
};

class PERSONS: public ARRAY{
public:
	PERSON *getitem(int no) const{
		return (PERSON*)ARRAY::getitem(no);
	}
};


static void sample_records()
{
	// Fill few records
	PERSONS tb;
	tb.add (new PERSON("Gelinas","Jacques","111-2222"));
	tb.add (new PERSON("Who","John","222-2222"));
	tb.add (new PERSON("Red","Jessy","333-2222"));
	//
	DIALOG_RECORDS dia;
	dia.newf_head ("","Name\tSurname\tTelephone");
	int nof = 0;
	while (1){
		for (int i=0; i<tb.getnb(); i++){
			PERSON *p = tb.getitem(i);
			char buf[100];
			snprintf (buf,sizeof(buf)-1,"%s\t%s",p->surname.get(),p->tel.get());
			dia.set_menuitem (i,p->name.get(),buf);
		}
		// Remove extra fields
		// dia.getnb() includes the header
		dia.remove_last (tb.getnb()+1);
		MENU_STATUS code = dia.editmenu ("title","introduction",help_nil
			,nof,MENUBUT_ADD|MENUBUT_DEL);
		if (code == MENU_QUIT || code == MENU_ESCAPE){
			break;
		}else if (code == MENU_DEL){
			tb.remove_del (0);
		}else if (code == MENU_ADD){
			static int add=1;	// Add arbitrary fields
			char buf[10];
			sprintf (buf,"%d",add++);
			tb.add (new PERSON(buf,buf,buf));
		}else{
			xconf_notice ("Edit field %d",nof);
		}
	}
}

// #Specend:
// #Specbeg: dialog / sample code / inter dialog messaging
/*
	This creates two independant dialogs which are exchanging
	messages. In this example, the same code is used to create
	the dialog, except that we exchange the message sent and received
	by each dialog.
*/


static void sample_dialog(PRIVATE_MESSAGE &send, PRIVATE_MESSAGE &receive)
{
	DIALOG dia;
	dia.waitfor (receive);
	SSTRING s;
	dia.newf_str ("A string",s);
	dia.setbutinfo (MENU_USR1,"Send","Send");
	int nof = 0;
	while (1){
		MENU_STATUS code = dia.edit ("title","The field grows"
			,help_nil,nof
			,MENUBUT_CANCEL|MENUBUT_ACCEPT|MENUBUT_USR1);
		if (code == MENU_CANCEL || code == MENU_ESCAPE){
			break;
		}else if (code == MENU_MESSAGE){
			if (dialog_testmessage(receive)){
				dia.save();
				s.append("X");
				dia.reload ();
			}
		}else if (code == MENU_USR1){
			dialog_sendmessage (send);
		}else{
			// Do nothing with s, just exit
			break;
		}
	}
}
static PRIVATE_MESSAGE p1,p2;

static void sample_fct (void *)
{
	sample_dialog (p2,p1);
}
static void sample_messages()
{
	uithread (sample_fct,NULL);
	sample_dialog (p1,p2);
}

// #Specend:

// #Specbeg: dialog / sample code / private messaging
/*
	This creates 4 dialogs, grouped two by two. Each dialog
	may send a message to its corresponding dialog. The two
	pairs are independant. Yet they are using exactly the same
	code. This shows how PRIVATE_MESSAGE may be used to
	achieve private communication.
*/

struct PMSGS{
	PRIVATE_MESSAGE p1;
	PRIVATE_MESSAGE p2;
};

static void sample_fct1 (void *data)
{
	PMSGS *p = (PMSGS*)data;
	sample_dialog (p->p2,p->p1);
}
static void sample_fct2 (void *data)
{
	PMSGS *p = (PMSGS*)data;
	sample_dialog (p->p1,p->p2);
}
/*
	This function creates two independant dialog which are exchanging
	messages.
*/
static void sample_privates()
{
	PMSGS group1;
	PMSGS group2;
	uithread (sample_fct1,&group1);
	uithread (sample_fct2,&group1);
	uithread (sample_fct1,&group2);
	sample_dialog (group2.p1,group2.p2);
}

// #Specend:


// #Specbeg: dialog / sample code / timers

/*
	This dialog presents a single numeric field, which grows by
	itself every 2 seconds
*/
static void sample_timer()
{
	DIALOG dia;
	int num = 0;
	dia.newf_num ("Some value",num);
	PRIVATE_MESSAGE msg;
	dialog_settimer(msg,2,false);
	dia.waitfortimer (msg);
	dia.setbutinfo (MENU_USR1,"stop timer","stop timer");
	dia.setbutinfo (MENU_USR2,"start timer","start timer");
	int nof=0;
	while (1){
		MENU_STATUS code = dia.edit ("title","The field grows"
			,help_nil,nof
			,MENUBUT_CANCEL|MENUBUT_ACCEPT|MENUBUT_USR1|MENUBUT_USR2);
		if (code == MENU_CANCEL || code == MENU_ESCAPE){
			break;
		}else if (code == MENU_MESSAGE){
			if (dialog_testtimer(msg)){
				num++;
				dia.reload ();
			}
		}else if (code == MENU_USR1){
			dialog_deltimer(msg);
		}else if (code == MENU_USR2){
			dialog_settimer(msg,2,true);
		}else{
			xconf_notice ("num = %d",num);
			break;
		}
	}
	dialog_deltimer(msg);
}
// #Specend:

// #Specbeg: dialog / sample code / reloading

/*
	This dialog shows how you can change the content of a field
	while the dialog is running. 

	The dialog present 5 check boxes, initially un-selected.
	Two extra buttons are added: All and None. The All button
	selects all check-boxes and the None button un-select them.

	Note that the DIALOG::reload() function accept 0 or one argument.
	The 0 argument version reloads all field input buffer from their
	corresponding variable while the other one reload a single field.
*/
static void sample_reload()
{
	DIALOG dia;
	char tb[5];
	for (int i=0; i<5; i++){
		char opt[20];
		sprintf (opt,"Option %d",i+1);
		tb[i] = 0;
		dia.newf_chk (opt,tb[i],"is active");
	}
	dia.setbutinfo (MENU_USR1,"All","All");
	dia.setbutinfo (MENU_USR2,"None","None");
	int nof = 0;
	while (1){
		MENU_STATUS code = dia.edit ("title","",help_nil
			,nof,MENUBUT_ACCEPT|MENUBUT_CANCEL|MENUBUT_USR1|MENUBUT_USR2);
		if (code == MENU_CANCEL || code == MENU_ESCAPE){
			break;
		}else if (code == MENU_USR1 || code == MENU_USR2){
			memset (tb,code == MENU_USR1 ? 1 : 0, sizeof(tb));
			dia.reload();
		}else{
			break;
		}
	}
}
		
// #Specend:
// #Specbeg: dialog / sample code / UI threads
/*
	Linuxconf UI toolkit is based on classical programming. This is
	sometime called modal programming. This kind of programming is
	much simpler as opposed to event driven (such as most GUI toolkit).
	It can be presented by the following pseudo code:

	#
	setup input field
	while (1){
		call the edit function
		process the user actions (button press)
	}
	#

	In a single function, one can create and handle a complete
	multi-field dialog. Much more simpler (code efficient).

	But it has a drawback. Once a dialog has called the edit function
	the program is locked there. So you can't have multiple dialog
	operating at once. This limitation has been solved using
	the User Interface Threads (UI threads).

	UI threads are keeping the simplicity of classical programming
	while removing the major limitation. This has been used
	primarily for the linuxconf project. Linuxconf is made
	of many many dialogs, so it pays to have this as simple as possible.
	We believe that this strategy could be used for more GUI oriented
	application as well though.

	The following applicaiton shows two independant contexts working
	at once.
*/
static void sample_context()
{
	while (1){
		if (dialog_yesno("title","Are you sure",help_nil)==MENU_YES){
			xconf_notice("Really");
		}else{
			xconf_notice("This shows");
			break;
		}
	}
}

static void sample_thread()
{
	uithread(sample_context);
	sample_context();
}
// #Specend:

int main (int argc, char *argv[])
{
	argc = dialog_parseuioptions (argc, argv);
	linuxconf_loadmsg ("linuxconf",PACKAGE_REV);
	translat_checkmissing();
	dialog_clear();
	if (argc == 2 && strcmp(argv[1],"records")==0){
		sample_records();
	}else if (argc == 2 && strcmp(argv[1],"messages")==0){
		sample_messages();
	}else if (argc == 2 && strcmp(argv[1],"timers")==0){
		sample_timer();
	}else if (argc == 2 && strcmp(argv[1],"privates")==0){
		sample_privates();
	}else if (argc == 2 && strcmp(argv[1],"reload")==0){
		sample_reload();
	}else if (argc == 2 && strcmp(argv[1],"thread")==0){
		sample_thread();
	}else{
		fprintf (stderr,"/tmp/x records\n");
		fprintf (stderr,"/tmp/x timers\n");
		fprintf (stderr,"/tmp/x messages\n");
		fprintf (stderr,"/tmp/x privates\n");
		fprintf (stderr,"/tmp/x reload\n");
		fprintf (stderr,"/tmp/x thread\n");
	}
	return 0;
}

